-- -------------------------------------------------------------------------- --
-- GridIndicatorSidePlus by kunda                                             --
-- -------------------------------------------------------------------------- --

local addonName, prg = ...
if type(prg.L) ~= "table" then print("ERROR", addonName, "No localization table.") return end
for k, v in pairs(prg.L) do if type(v) ~= "string" then prg.L[k] = tostring(k) end end
local L = prg.L

local Grid = Grid
local GridFrame = Grid:GetModule("GridFrame")
local GridIndicatorSidePlus = GridFrame:NewModule("GridIndicatorSidePlus")
local configMode = false

local BACKDROP = {
	bgFile = "Interface\\BUTTONS\\WHITE8X8",
	edgeFile = "Interface\\BUTTONS\\WHITE8X8",
	tile = true, tileSize = 8, edgeSize = 1,
	insets = {left = 1, right = 1, top = 1, bottom = 1}
}
local FrameBackdrop = {
	bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background",
	edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Border",
	tile = true, tileSize = 32, edgeSize = 32,
	insets = {left = 8, right = 8, top = 8, bottom = 8}
}



local function NewIndicator(frame)
	local square = CreateFrame("Frame", nil, frame)
	square:SetBackdrop(BACKDROP)
	square:SetBackdropBorderColor(0, 0, 0, 1)
	return square
end



local function ResetIndicator(self)
	local frame = self.__owner
	local indicator = self.__id
	local AddonProfile = GridIndicatorSidePlus.db.profile
	local GridProfile = GridFrame.db.profile

	local space = AddonProfile.SidePlusSpace
	local offset = AddonProfile.SidePlusOffset

	local wh = AddonProfile.SidePlusSize
	if AddonProfile.SidePlusOriginalSize then
		wh = GridProfile.cornerSize
	end

	self:SetWidth(wh)
	self:SetHeight(wh)
	self:SetParent(frame.indicators.bar)
	self:SetFrameLevel(frame.indicators.bar:GetFrameLevel() + 1)

	self:ClearAllPoints()
	if     indicator == "sidePlusTleft"   then self:SetPoint("TOP", frame, "TOP", -space, offset*-1)
	elseif indicator == "sidePlusTcenter" then self:SetPoint("TOP", frame, "TOP", 0, offset*-1)
	elseif indicator == "sidePlusTright"  then self:SetPoint("TOP", frame, "TOP", space, offset*-1)
	elseif indicator == "sidePlusBleft"   then self:SetPoint("BOTTOM", frame, "BOTTOM", -space, offset)
	elseif indicator == "sidePlusBcenter" then self:SetPoint("BOTTOM", frame, "BOTTOM", 0, offset)
	elseif indicator == "sidePlusBright"  then self:SetPoint("BOTTOM", frame, "BOTTOM", space, offset)
	elseif indicator == "sidePlusLtop"    then self:SetPoint("LEFT", frame, "LEFT", offset, space)
	elseif indicator == "sidePlusLcenter" then self:SetPoint("LEFT", frame, "LEFT", offset, 0)
	elseif indicator == "sidePlusLbottom" then self:SetPoint("LEFT", frame, "LEFT", offset, -space)
	elseif indicator == "sidePlusRtop"    then self:SetPoint("RIGHT", frame, "RIGHT", offset*-1, space)
	elseif indicator == "sidePlusRcenter" then self:SetPoint("RIGHT", frame, "RIGHT", offset*-1, 0)
	elseif indicator == "sidePlusRbottom" then self:SetPoint("RIGHT", frame, "RIGHT", offset*-1, -space)
	end
end



local function SetIndicator(self, color, text, value, maxValue, texture, texCoords, count, start, duration)
	if not color then color = { r = 1, g = 1, b = 1, a = 1 } end
	self:SetBackdropColor(color.r or 1, color.g or 1, color.b or 1, color.a or 1)
	self:Show()
end



local function ClearIndicator(self)
	self:SetBackdropColor(1, 1, 1, 1)
	self:Hide()
end



local function SetCornerSize(frame)
	ResetIndicator(frame.indicators.sidePlusTleft)
	ResetIndicator(frame.indicators.sidePlusTcenter)
	ResetIndicator(frame.indicators.sidePlusTright)
	ResetIndicator(frame.indicators.sidePlusBleft)
	ResetIndicator(frame.indicators.sidePlusBcenter)
	ResetIndicator(frame.indicators.sidePlusBright)
	ResetIndicator(frame.indicators.sidePlusLtop)
	ResetIndicator(frame.indicators.sidePlusLcenter)
	ResetIndicator(frame.indicators.sidePlusLbottom)
	ResetIndicator(frame.indicators.sidePlusRtop)
	ResetIndicator(frame.indicators.sidePlusRcenter)
	ResetIndicator(frame.indicators.sidePlusRbottom)
end



GridIndicatorSidePlus.defaultDB = {
	SidePlusSpace = 8,
	SidePlusSize = 5,
	SidePlusOriginalSize = true,
	SidePlusOffset = 1,
}



local options = {
	type = "group",
	--inline = true,
	icon = "Interface\\AddOns\\GridIndicatorSidePlus\\GridIndicatorSidePlus-icon-TRBL",
	name = L["Side Plus"],
	desc = L["Options for Side Plus."],
	order = -0.561,--460,
	args = {
		["configuration"] = {
			type = "toggle",
			name = L["Configuration Mode"],
			desc = L["Shows all Side Plus indicators."],
			order = 10,
			get = function(self) return configMode end,
			set = function(_, v)
				configMode = v
				GridIndicatorSidePlus:ConfigMode()
			end
		},
		["header1"] = {
			type = "header",
			order = 15,
			width = "full",
			name = "",
		},
		["sideplusspace"] = {
			type = "range",
			name = L["Side Plus Space"],
			desc = L["Adjust the space between the Side Plus indicators."],
			order = 20,
			min = 0,
			max = 40,
			step = 1,
			get = function() return GridIndicatorSidePlus.db.profile.SidePlusSpace end,
			set = function(_, v)
				GridIndicatorSidePlus.db.profile.SidePlusSpace = v
				for _, f in pairs(GridFrame.registeredFrames) do
					SetCornerSize(f)
				end
			end,
		},
		["sideplusoffset"] = {
			type = "range",
			name = L["Side Plus Offset"],
			desc = L["Adjust the offset for the Side Plus indicators."],
			order = 30,
			min = -20,
			max = 20,
			step = 1,
			get = function() return GridIndicatorSidePlus.db.profile.SidePlusOffset end,
			set = function(_, v)
				GridIndicatorSidePlus.db.profile.SidePlusOffset = v
				for _, f in pairs(GridFrame.registeredFrames) do
					SetCornerSize(f)
				end
			end,
		},
		["sideplusoriginalsize"] = {
			type = "toggle",
			name = L["Same size as Grid standard"],
			desc = L["If enabled, the size of the Side Plus indicator is adjustable with the standard Grid corner option. If deactivated, you can set an individual size for the Side Plus indicators."],
			order = 40,
			width = "full",
			get = function() return GridIndicatorSidePlus.db.profile.SidePlusOriginalSize end,
			set = function(_, v)
				GridIndicatorSidePlus.db.profile.SidePlusOriginalSize = v
				for _, f in pairs(GridFrame.registeredFrames) do
					SetCornerSize(f)
				end
			end,
		},
		["sideplussize"] = {
			type = "range",
			name = L["Side Plus Size"],
			desc = L["Adjust the size of the Side Plus indicators."],
			order = 50,
			disabled = function() return GridIndicatorSidePlus.db.profile.SidePlusOriginalSize end,
			min = 3,
			max = 20,
			step = 1,
			get = function() return GridIndicatorSidePlus.db.profile.SidePlusSize end,
			set = function(_, v)
				GridIndicatorSidePlus.db.profile.SidePlusSize = v
				for _, f in pairs(GridFrame.registeredFrames) do
					SetCornerSize(f)
				end
			end,
		}
	}
}
Grid.options.args["GridIndicatorSidePlus"] = options



local statusmap = GridFrame.db.profile.statusmap
if not statusmap["sidePlusTleft"] then
	statusmap["sidePlusTleft"] = {}
	statusmap["sidePlusTcenter"] = {}
	statusmap["sidePlusTright"] = {}
	statusmap["sidePlusBleft"] = {}
	statusmap["sidePlusBcenter"] = {}
	statusmap["sidePlusBright"] = {}
	statusmap["sidePlusLtop"] = {}
	statusmap["sidePlusLcenter"] = {}
	statusmap["sidePlusLbottom"] = {}
	statusmap["sidePlusRtop"] = {}
	statusmap["sidePlusRcenter"] = {}
	statusmap["sidePlusRbottom"] = {}
end



function GridIndicatorSidePlus:OnInitialize()
	if not self.db then
		self.db = Grid.db:RegisterNamespace(self.moduleName, { profile = self.defaultDB or { } })
	end
	GridFrame:RegisterIndicator("sidePlusTleft",   L["left"],   NewIndicator, ResetIndicator, SetIndicator, ClearIndicator)
	GridFrame:RegisterIndicator("sidePlusTcenter", L["center"], NewIndicator, ResetIndicator, SetIndicator, ClearIndicator)
	GridFrame:RegisterIndicator("sidePlusTright",  L["right"],  NewIndicator, ResetIndicator, SetIndicator, ClearIndicator)
	GridFrame:RegisterIndicator("sidePlusBleft",   L["left"],   NewIndicator, ResetIndicator, SetIndicator, ClearIndicator)
	GridFrame:RegisterIndicator("sidePlusBcenter", L["center"], NewIndicator, ResetIndicator, SetIndicator, ClearIndicator)
	GridFrame:RegisterIndicator("sidePlusBright",  L["right"],  NewIndicator, ResetIndicator, SetIndicator, ClearIndicator)
	GridFrame:RegisterIndicator("sidePlusLtop",    L["top"],    NewIndicator, ResetIndicator, SetIndicator, ClearIndicator)
	GridFrame:RegisterIndicator("sidePlusLcenter", L["center"], NewIndicator, ResetIndicator, SetIndicator, ClearIndicator)
	GridFrame:RegisterIndicator("sidePlusLbottom", L["bottom"], NewIndicator, ResetIndicator, SetIndicator, ClearIndicator)
	GridFrame:RegisterIndicator("sidePlusRtop",    L["top"],    NewIndicator, ResetIndicator, SetIndicator, ClearIndicator)
	GridFrame:RegisterIndicator("sidePlusRcenter", L["center"], NewIndicator, ResetIndicator, SetIndicator, ClearIndicator)
	GridFrame:RegisterIndicator("sidePlusRbottom", L["bottom"], NewIndicator, ResetIndicator, SetIndicator, ClearIndicator)

	hooksecurefunc(GridFrame, "UpdateOptionsMenu", self.ChangeOptionsMenu)
	GridIndicatorSidePlus:ChangeOptionsMenu()
	GridIndicatorSidePlus:CreateConfigFrame()
end



function GridIndicatorSidePlus:OnEnable()
end



function GridIndicatorSidePlus:OnDisable()
end



function GridIndicatorSidePlus:Reset()
end



function GridIndicatorSidePlus:ChangeOptionsMenu()
	if not GridIndicatorSidePlus:IsEnabled() then return end
	if not Grid.options then return end
	if not Grid.options.args then return end
	local GridIndicator
	if     type(Grid.options.args.GridIndicator) == "table" then GridIndicator = "GridIndicator"
	elseif type(Grid.options.args.Indicators)    == "table" then GridIndicator = "Indicators" end
	if not Grid.options.args[GridIndicator] then return end
	if not Grid.options.args[GridIndicator].args then return end

	if not Grid.options.args[GridIndicator].args.sidePlusTleft then return end

	Grid.options.args[GridIndicator].args.sideplusT = {
		type = "group",
		icon = "Interface\\AddOns\\GridIndicatorSidePlus\\GridIndicatorSidePlus-icon-T",
		name = L["Side Plus"].." ("..L["Top Side"]..")",
		desc = L["Options for Top Side indicators."],
		order = 16.1,
		args = {
			["sidePlusTleft"]   = Grid.options.args[GridIndicator].args.sidePlusTleft,
			["sidePlusTcenter"] = Grid.options.args[GridIndicator].args.sidePlusTcenter,
			["sidePlusTright"]  = Grid.options.args[GridIndicator].args.sidePlusTright
		}
	}
	Grid.options.args[GridIndicator].args.sideplusT.args.sidePlusTleft.icon = "Interface\\AddOns\\GridIndicatorSidePlus\\GridIndicatorSidePlus-icon-Tleft"
	Grid.options.args[GridIndicator].args.sideplusT.args.sidePlusTcenter.icon = "Interface\\AddOns\\GridIndicatorSidePlus\\GridIndicatorSidePlus-icon-Tcenter"
	Grid.options.args[GridIndicator].args.sideplusT.args.sidePlusTright.icon = "Interface\\AddOns\\GridIndicatorSidePlus\\GridIndicatorSidePlus-icon-Tright"
	Grid.options.args[GridIndicator].args.sideplusT.args.sidePlusTleft.order = 1
	Grid.options.args[GridIndicator].args.sideplusT.args.sidePlusTcenter.order = 2
	Grid.options.args[GridIndicator].args.sideplusT.args.sidePlusTright.order = 3
	Grid.options.args[GridIndicator].args.sidePlusTleft = nil
	Grid.options.args[GridIndicator].args.sidePlusTcenter = nil
	Grid.options.args[GridIndicator].args.sidePlusTright = nil

	Grid.options.args[GridIndicator].args.sideplusB = {
		type = "group",
		icon = "Interface\\AddOns\\GridIndicatorSidePlus\\GridIndicatorSidePlus-icon-B",
		name = L["Side Plus"].." ("..L["Bottom Side"]..")",
		desc = L["Options for Bottom Side indicators."],
		order = 17.1,
		args = {
			["sidePlusBleft"]   = Grid.options.args[GridIndicator].args.sidePlusBleft,
			["sidePlusBcenter"] = Grid.options.args[GridIndicator].args.sidePlusBcenter,
			["sidePlusBright"]  = Grid.options.args[GridIndicator].args.sidePlusBright
		}
	}
	Grid.options.args[GridIndicator].args.sideplusB.args.sidePlusBleft.icon = "Interface\\AddOns\\GridIndicatorSidePlus\\GridIndicatorSidePlus-icon-Bleft"
	Grid.options.args[GridIndicator].args.sideplusB.args.sidePlusBcenter.icon = "Interface\\AddOns\\GridIndicatorSidePlus\\GridIndicatorSidePlus-icon-Bcenter"
	Grid.options.args[GridIndicator].args.sideplusB.args.sidePlusBright.icon = "Interface\\AddOns\\GridIndicatorSidePlus\\GridIndicatorSidePlus-icon-Bright"
	Grid.options.args[GridIndicator].args.sideplusB.args.sidePlusBleft.order = 1
	Grid.options.args[GridIndicator].args.sideplusB.args.sidePlusBcenter.order = 2
	Grid.options.args[GridIndicator].args.sideplusB.args.sidePlusBright.order = 3
	Grid.options.args[GridIndicator].args.sidePlusBleft = nil
	Grid.options.args[GridIndicator].args.sidePlusBcenter = nil
	Grid.options.args[GridIndicator].args.sidePlusBright = nil

	Grid.options.args[GridIndicator].args.sideplusL = {
		type = "group",
		icon = "Interface\\AddOns\\GridIndicatorSidePlus\\GridIndicatorSidePlus-icon-L",
		name = L["Side Plus"].." ("..L["Left Side"]..")",
		desc = L["Options for Left Side indicators."],
		order = 18.1,
		args = {
			["sidePlusLtop"]    = Grid.options.args[GridIndicator].args.sidePlusLtop,
			["sidePlusLcenter"] = Grid.options.args[GridIndicator].args.sidePlusLcenter,
			["sidePlusLbottom"] = Grid.options.args[GridIndicator].args.sidePlusLbottom
		}
	}
	Grid.options.args[GridIndicator].args.sideplusL.args.sidePlusLtop.icon = "Interface\\AddOns\\GridIndicatorSidePlus\\GridIndicatorSidePlus-icon-Ltop"
	Grid.options.args[GridIndicator].args.sideplusL.args.sidePlusLcenter.icon = "Interface\\AddOns\\GridIndicatorSidePlus\\GridIndicatorSidePlus-icon-Lcenter"
	Grid.options.args[GridIndicator].args.sideplusL.args.sidePlusLbottom.icon = "Interface\\AddOns\\GridIndicatorSidePlus\\GridIndicatorSidePlus-icon-Lbottom"
	Grid.options.args[GridIndicator].args.sideplusL.args.sidePlusLtop.order = 1
	Grid.options.args[GridIndicator].args.sideplusL.args.sidePlusLcenter.order = 2
	Grid.options.args[GridIndicator].args.sideplusL.args.sidePlusLbottom.order = 3
	Grid.options.args[GridIndicator].args.sidePlusLtop = nil
	Grid.options.args[GridIndicator].args.sidePlusLcenter = nil
	Grid.options.args[GridIndicator].args.sidePlusLbottom = nil

	Grid.options.args[GridIndicator].args.sideplusR = {
		type = "group",
		icon = "Interface\\AddOns\\GridIndicatorSidePlus\\GridIndicatorSidePlus-icon-R",
		name = L["Side Plus"].." ("..L["Right Side"]..")",
		desc = L["Options for Right Side indicators."],
		order = 19.1,
		args = {
			["sidePlusRtop"]    = Grid.options.args[GridIndicator].args.sidePlusRtop,
			["sidePlusRcenter"] = Grid.options.args[GridIndicator].args.sidePlusRcenter,
			["sidePlusRbottom"] = Grid.options.args[GridIndicator].args.sidePlusRbottom
		}
	}
	Grid.options.args[GridIndicator].args.sideplusR.args.sidePlusRtop.icon = "Interface\\AddOns\\GridIndicatorSidePlus\\GridIndicatorSidePlus-icon-Rtop"
	Grid.options.args[GridIndicator].args.sideplusR.args.sidePlusRcenter.icon = "Interface\\AddOns\\GridIndicatorSidePlus\\GridIndicatorSidePlus-icon-Rcenter"
	Grid.options.args[GridIndicator].args.sideplusR.args.sidePlusRbottom.icon = "Interface\\AddOns\\GridIndicatorSidePlus\\GridIndicatorSidePlus-icon-Rbottom"
	Grid.options.args[GridIndicator].args.sideplusR.args.sidePlusRtop.order = 1
	Grid.options.args[GridIndicator].args.sideplusR.args.sidePlusRcenter.order = 2
	Grid.options.args[GridIndicator].args.sideplusR.args.sidePlusRbottom.order = 3
	Grid.options.args[GridIndicator].args.sidePlusRtop = nil
	Grid.options.args[GridIndicator].args.sidePlusRcenter = nil
	Grid.options.args[GridIndicator].args.sidePlusRbottom = nil
end



function GridIndicatorSidePlus:CreateConfigFrame()
	GridIndicatorSidePlus.ConfigWarningFrame = CreateFrame("Frame", nil, UIParent)
	local frame = GridIndicatorSidePlus.ConfigWarningFrame
	frame:EnableMouse(true)
	frame:SetMovable(true)
	frame:SetToplevel(true)
	frame:SetClampedToScreen(true)
	frame:SetHeight(100)
	frame:SetBackdrop(FrameBackdrop)
	frame:SetPoint("LEFT", GridLayoutFrame, "RIGHT", 40, 0)
	frame:SetScript("OnMouseDown", function(self)
		self.isMoving = true
		self:StartMoving()
	end)
	frame:SetScript("OnMouseUp", function(self)
		if not self.isMoving then return end
		self.isMoving = nil
		self:StopMovingOrSizing()
	end)
	frame:Hide()

	local background = frame:CreateTexture(nil, "ARTWORK")
	background:SetPoint("TOP", 0, -15)
	background:SetTexture("Interface\\DialogFrame\\UI-Dialog-Icon-AlertNew")
	background:SetHeight(24)
	background:SetWidth(24)

	local txt = frame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	txt:SetPoint("TOP", background, "BOTTOM", 0, -5)
	txt:SetText("Grid - "..L["Side Plus"].." - "..L["Configuration Mode"])
	txt:SetTextColor(1, 1, 1, 1)

	local button = CreateFrame("Button", nil, frame, "UIPanelButtonTemplate")
	button:SetHeight(24)
	button:SetPoint("TOP", txt, "BOTTOM", 0, -5)
	button:SetText(CLOSE)
	button:SetScript("OnClick", function()
		configMode = false
		GridIndicatorSidePlus:ConfigMode()
	end)

	local w = txt:GetStringWidth()
	button:SetWidth(w+50)
	frame:SetWidth(w+50+60)
end



function GridIndicatorSidePlus:ConfigMode()
	if configMode then
		GridIndicatorSidePlus.ConfigWarningFrame:Show()
		for _, frame in pairs(GridFrame.registeredFrames) do
			SetIndicator(frame.indicators.sidePlusTleft,   {r=0.31, g=0.62, b=0.86, a=1})
			SetIndicator(frame.indicators.sidePlusTcenter, {r=0.74, g=0.34, b=0.49, a=1})
			SetIndicator(frame.indicators.sidePlusTright,  {r=0.12, g=0.46, b=0.97, a=1})
			SetIndicator(frame.indicators.sidePlusBleft,   {r=0.77, g=0.02, b=0.24, a=1})
			SetIndicator(frame.indicators.sidePlusBcenter, {r=0.17, g=0.65, b=0.33, a=1})
			SetIndicator(frame.indicators.sidePlusBright,  {r=0.56, g=0.74, b=0.81, a=1})
			SetIndicator(frame.indicators.sidePlusLtop,    {r=0.87, g=0.71, b=0.06, a=1})
			SetIndicator(frame.indicators.sidePlusLcenter, {r=0.29, g=0.83, b=0.36, a=1})
			SetIndicator(frame.indicators.sidePlusLbottom, {r=0.59, g=0.31, b=0.40, a=1})
			SetIndicator(frame.indicators.sidePlusRtop,    {r=0.42, g=0.77, b=0.11, a=1})
			SetIndicator(frame.indicators.sidePlusRcenter, {r=0.33, g=0.65, b=0.37, a=1})
			SetIndicator(frame.indicators.sidePlusRbottom, {r=0.89, g=0.57, b=0.27, a=1})
		end
	else
		GridIndicatorSidePlus.ConfigWarningFrame:Hide()
		for _, frame in pairs(GridFrame.registeredFrames) do
			ClearIndicator(frame.indicators.sidePlusTleft)
			ClearIndicator(frame.indicators.sidePlusTcenter)
			ClearIndicator(frame.indicators.sidePlusTright)
			ClearIndicator(frame.indicators.sidePlusBleft)
			ClearIndicator(frame.indicators.sidePlusBcenter)
			ClearIndicator(frame.indicators.sidePlusBright)
			ClearIndicator(frame.indicators.sidePlusLtop)
			ClearIndicator(frame.indicators.sidePlusLcenter)
			ClearIndicator(frame.indicators.sidePlusLbottom)
			ClearIndicator(frame.indicators.sidePlusRtop)
			ClearIndicator(frame.indicators.sidePlusRcenter)
			ClearIndicator(frame.indicators.sidePlusRbottom)
		end
	end
end